@@ -3,6 +3,11 @@ from nautobot.extras.plugins import PluginMenuButton, PluginMenuItem
 from nautobot.utilities.choices import ButtonColorChoices
 
 menu_items = (
+    PluginMenuItem(
+        link="plugins:nautobot_circuit_maintenance:circuitmaintenance_overview",
+        link_text="Dashboard",
+        permissions=["nautobot_circuit_maintenance.view_circuitmaintenance"],
+    ),
     PluginMenuItem(
         link="plugins:nautobot_circuit_maintenance:circuitmaintenance_list",
         link_text="Circuit Maintenances",
